==> 1.0.zip: bean/main.c <==
/////////////////
// CUSTOM HEADERS
// ORDER MATTERS !!!
#include "jsystem.h"
#include "CSV/CSV.h"
#include "BDL/BDL.h"
#include "BTP/BIN_BTP.h"
#include "BTP/CSV_BTP.h"


/////////////////////////
// ENUM WITH ACTION TO DO
enum PROGRAM_ACTION {
    
    // nothing to do
    NOTHING_TO_DO,
    
    // checking
    CHECK_ANIM,             // check if anim file has a correct structure
    CHECK_MODEL,            // check if model file has a correct structure
    CHECK_ANIM_WITH_MODEL,  // check if anim file (either CSV or BIN) and binary model file are related
    
    // reading
    READ_ANIM,              // read anim file (either CSV or BIN)
    READ_MODEL,             // read binary model file
    READ_ANIM_WITH_MODEL,   // read both anim file (either CSV or BIN) and binary model file
    
    // converting
    CONVERT_CSV_ANIM_TO_BIN_ANIM,   // convert csv anim file into bin anim file
    CONVERT_BIN_ANIM_TO_CSV_ANIM,   // convert bin anim file into csv anim file
  };


////////////////////////
// VALID COMMAND STRINGS
u1 ANIM_INPUT_ARG[] = "-anim";
u1 MODEL_INPUT_ARG[] = "-model";
u1 READ_FILE_ARG[] = "-read";
u1 CHECK_FILE_ARG[] = "-check";
u1 CONVERT_FILE_ARG[] = "-convert";
u1 CONVERT_TO_CSV[] = "csv";
u1 CONVERT_TO_BIN[] = "bin";


////////////////
// PROGRAM USAGE
// ./a.out -file [JSYSTEM animation file] -model [BDL model] -read/-convert [csv/bin]

// functions only used on main.c
void display_help();
int check_command(int, char * []);
int is_anim(char *);
u4 is_model(char *);
void read_anim(char *);
void read_model(char *);
void write_csv_anim(char *, int);
void write_bin_anim(char *, int, char *);


////////////////
// MAIN FUNCTION
////////////////

int main(int argc, char * argv[])
{ 
  // check command and get program action
  enum PROGRAM_ACTION ACTION = check_command(argc, argv);
  
  enum ANIM_FILE_TYPE ANIM_TYPE = NO_TYPE;
  
  // another epic switch statement
  switch(ACTION)
  {
    ////////////////////////////////////
    // ./a.out -anim iceflick.btp -check
    case(CHECK_ANIM):
      is_anim(argv[2]);     
      break;
    
    //////////////////////////////////
    // ./a.out -model mario.bdl -check
    case(CHECK_MODEL):
      is_model(argv[2]);
      break;
    
    ///////////////////////////////////
    // ./a.out -anim iceflick.btp -read
    case(READ_ANIM):
    
      // read anim file
      read_anim(argv[2]);
      break;
    
    /////////////////////////////////  
    // ./a.out -model mario.bdl -read
    case(READ_MODEL):
      
      // read model
      read_model(argv[2]);
      break; 
    
    /////////////////////////////////////////////////////
    // ./a.out -anim iceflick.btp -model mario.bdl -check
    case(CHECK_ANIM_WITH_MODEL):
      
      // check anim file and model file
      ANIM_TYPE = is_anim(argv[2]);            
      is_model(argv[4]);
      
      // decide what to do with the anim file type
      if (ANIM_TYPE == BIN_BTP_TYPE) {
        if (CHECK_BIN_BTP_WITH_BDL(argv[2], argv[4]) == false)
          exit(1);
      }
      else if (ANIM_TYPE == CSV_BTP_TYPE) {
        if (CHECK_CSV_BTP_WITH_BDL(argv[2], argv[4]) == false)
          exit(1);
      }
      break; 
    
    ////////////////////////////////////////////////////
    // ./a.out -anim iceflick.btp -model mario.bdl -read
    case(READ_ANIM_WITH_MODEL):
      
      // read both files
      read_model(argv[4]);
      read_anim(argv[2]);
      break;
    
    ///////////////////////////////////////////////////////////
    // ./a.out -anim iceflick.btp -convert csv
    // ./a.out -anim iceflick.btp -model mario.bdl -convert csv
    case(CONVERT_BIN_ANIM_TO_CSV_ANIM):
      
      // if model is provided
      // check if both anim and model files are related      
      // if so write the CSV file
      // otherwise just write the CSV BTP
      
      // check anim file
      ANIM_TYPE = is_anim(argv[2]); 
      
      // check if the file is actually valid for conversion
      if (ANIM_TYPE == CSV_BTP_TYPE)
      {
        printf("\nCannot convert CSV file into CSV file.\nTerminating program.\n");
        exit(1);
      }
      
      // model is provided
      if (argc == 7)
      {
        // check model file           
        is_model(argv[4]);
        
        // decide what to do with the anim file type
        if (ANIM_TYPE == BIN_BTP_TYPE) {
          if (CHECK_BIN_BTP_WITH_BDL(argv[2], argv[4]) == false)
            exit(1);
        }
      }
      
      // write csv animation file
      write_csv_anim(argv[2], ANIM_TYPE);
      break;
    
    ///////////////////////////////////////////////////////////////
    // ./a.out -anim iceflick.btp.csv -model mario.bdl -convert bin
    case(CONVERT_CSV_ANIM_TO_BIN_ANIM):
      
      // check anim file
      ANIM_TYPE = is_anim(argv[2]); 
      
      // check if the file is actually valid for conversion
      if (ANIM_TYPE == BIN_BTP_TYPE)
      {
        printf("\nCannot convert BIN file into BIN file.\nTerminating program.\n");
        exit(1);
      }
      
      // check model file           
      is_model(argv[4]);
        
      // decide what to do with the anim file type
      if (ANIM_TYPE == CSV_BTP_TYPE)
      {
        if (CHECK_CSV_BTP_WITH_BDL(argv[2], argv[4]) == false)
          exit(1);
      }
      
      // write csv animation file
      write_bin_anim(argv[2], ANIM_TYPE, argv[4]);      
      break;
    
    ///////
    // how?
    default:
      printf("How on earth you managed to get here?\n");
      break;
  }
  
  // program end
  return 0;
}


//////////////////////////
// display_help() function
// function used to display program usage in case
// user inputs wrong information to the program.
// Also terminates the program session.
void display_help()
{
  // display some help  
  printf("Mateee... I dunnae know what to do with that command <:[\n");
  printf("Try again after these command examples:\n");
  
  printf("\n### Check model file\n");
  printf("bean -model mario.bdl -check\n");
  
  printf("\n### Check anim file\n");
  printf("bean -anim iceflick.btp -check\n");
  
  printf("\n### Read model file file\n");
  printf("bean -model mario.bdl -read\n");
  
  printf("\n### Read anim file file\n");
  printf("bean -anim iceflick.btp -read\n");
  
  printf("\n### Convert a BIN anim file into a CSV (NOT providing a model file)\n");
  printf("bean -anim iceflick.btp -convert csv\n");
  
  printf("\n### Read both an anim file and a model file\n");
  printf("bean -anim iceflick.btp -model mario.bdl -read\n");
  
  printf("\n### Check if an anim file and a model file are related\n");
  printf("bean -anim iceflick.btp -model mario.bdl -check\n");
  
  printf("\n### Convert a BIN anim file into a CSV (providing a model file)\n");
  printf("bean -anim iceflick.btp -model mario.bdl -convert csv\n");
  
  printf("\n### Convert a CSV anim file into a BIN (providing a model file)\n");
  printf("bean -anim iceflick.btp.csv -model mario.bdl -convert bin\n");
  
  // incorrect command exit
  exit(1);
}


///////////////////////////
// check_command() function
// function used to check if command 
// entered by the user is valid. Will
// call display_help() in case it isn't.
int check_command(int argc, char * argv[])
{
  // variable to return
  enum PROGRAM_ACTION ACTION;
  
  // huge and epic switch statement >:]
  switch (argc)
  {
    // - checking a single file (anim or model)
    // - reading a single file (anim or model)
    
    // ./a.out -model mario.bdl -check
    // ./a.out -anim iceflick.btp -read
    
    case(4):      
      
      // checking an anim file
      if (compare_strings(argv[1], ANIM_INPUT_ARG) && compare_strings(argv[3], CHECK_FILE_ARG))
        ACTION = CHECK_ANIM;
      // checking a model file
      else if (compare_strings(argv[1], MODEL_INPUT_ARG) && compare_strings(argv[3], CHECK_FILE_ARG))
        ACTION = CHECK_MODEL;
      // reading an anim file
      else if (compare_strings(argv[1], ANIM_INPUT_ARG) && compare_strings(argv[3], READ_FILE_ARG))
        ACTION = READ_ANIM;
      // reading an model file
      else if (compare_strings(argv[1], MODEL_INPUT_ARG) && compare_strings(argv[3], READ_FILE_ARG))
        ACTION = READ_MODEL;
      // invalid command
      else
        ACTION = NOTHING_TO_DO;
        
      break;
    
    // - converting a bin anim file into csv (without providing model file)
    
    // ./a.out -anim iceflick.btp -convert csv
    
    case(5):
      
      // converting a bin anim file into csv
      if (compare_strings(argv[1], ANIM_INPUT_ARG) &&
          compare_strings(argv[3], CONVERT_FILE_ARG) &&
          compare_strings(argv[4], CONVERT_TO_CSV))
        ACTION = CONVERT_BIN_ANIM_TO_CSV_ANIM;
      // invalid command
      else
        ACTION = NOTHING_TO_DO;
    
      break;
    
    // - checking if an anim file and a model file relate
    // - reading both an anim file and a model file (don't check if they relate)
    
    // ./a.out -anim iceflick.btp -model mario.bdl -read
    // ./a.out -anim iceflick.btp -model mario.bdl -check
    
    case(6):
    
      if (compare_strings(argv[1], ANIM_INPUT_ARG) &&
          compare_strings(argv[3], MODEL_INPUT_ARG))
      {
        // read both anim and model files
        if (compare_strings(argv[5], READ_FILE_ARG))
          ACTION = READ_ANIM_WITH_MODEL;
        // check if both anim and model files are related
        else if (compare_strings(argv[5], CHECK_FILE_ARG))
          ACTION = CHECK_ANIM_WITH_MODEL;
        // invalid command
        else
          ACTION = NOTHING_TO_DO;
      }
      // invalid command
      else
        ACTION = NOTHING_TO_DO;
    
      break;
    
    // - converting a binary anim file into a csv file (providing model file)
    // - converting a csv anim file into a bin file (providing model file)
    
    // ./a.out -anim iceflick.btp -model mario.bdl -convert csv
    // ./a.out -anim iceflick.btp.csv -model mario.bdl -convert bin
    
    case(7):
      
      if (compare_strings(argv[1], ANIM_INPUT_ARG) &&
          compare_strings(argv[3], MODEL_INPUT_ARG) &&
          compare_strings(argv[5], CONVERT_FILE_ARG))
      {
        // convert to csv
        if (compare_strings(argv[6], CONVERT_TO_CSV))
          ACTION = CONVERT_BIN_ANIM_TO_CSV_ANIM;
        // convert to bin
        else if (compare_strings(argv[6], CONVERT_TO_BIN))
          ACTION = CONVERT_CSV_ANIM_TO_BIN_ANIM;
        // invalid command
        else
          ACTION = NOTHING_TO_DO;
      }
      // invalid command
      else
        ACTION = NOTHING_TO_DO;
      
      break;
      
    default:
      ACTION = NOTHING_TO_DO;
      break;
  }
  
  if (ACTION == NOTHING_TO_DO)
    display_help();
  
  return ACTION;
}

////////////////////
// is_anim() function
// function used to check if a file is a correct anim file
int is_anim(char * anim_file_path)
{
  // enum integer to return
  enum ANIM_FILE_TYPE anim_type = NO_TYPE;
  
  // get file extension
  char * file_ext = get_file_ext(anim_file_path);
  
  // CSV file check
  if (compare_strings(file_ext, FILE_EXT[0]))
    anim_type = GET_CSV_TYPE(anim_file_path);
  // BIN BTP check
  else if (compare_strings(file_ext, FILE_EXT[2]))
    anim_type = CHECK_BIN_BTP(anim_file_path);
    
  if (anim_type == NO_TYPE)
  {
    printf("File %s is not a valid anim file.\nCheck the file extension and/or file contents and try again.\n", anim_file_path);
    exit(1);
  }
  
  // otherwise, return file size
  return anim_type;
}


////////////////////
// is_bdl() function
// function used to check if a file is a BDL file
u4 is_model(char * file_path)
{
  // get file size
  u4 bdl_size = CHECK_BDL(file_path);
  // get file extension
  char * file_ext = get_file_ext(file_path);
  
  // even if the file is good, the program won't accept extensionless files
  if (compare_strings(file_ext, FILE_EXT[1]) == false)
    bdl_size = 0;
  
   // CHECK_BDL returned 0 or the file extension is inconsistent
  if (bdl_size == false) {
    printf("File %s is not a valid BDL.\nCheck the file and try again.\n", file_path);
    exit(1);
  }
  
  // if the file is good, return its file size
  return bdl_size;
}


///////////////////////
// read_anim() function
// function to read an anim file based on the anim file type
void read_anim(char * anim_file_path)
{
  // check the anim file type and load it into memory
  int ANIM_TYPE = is_anim(anim_file_path);
  u1 * anim_file = load_file(anim_file_path);
  
  // decide what to do with the anim file type
  if (ANIM_TYPE == BIN_BTP_TYPE)
  {
    // create and fill CSV_BTP struct
    struct BIN_BTP BIN_BTP;
    BUILD_BIN_BTP_STRUCT(&BIN_BTP, anim_file);
    PRINT_BIN_BTP_STRUCT(&BIN_BTP, anim_file);
  }
  else if (ANIM_TYPE == CSV_BTP_TYPE)
  {
    // create and fill CSV_BTP struct
    struct CSV_BTP CSV_BTP;
    BUILD_CSV_BTP_STRUCT(&CSV_BTP, anim_file);
    PRINT_CSV_BTP_STRUCT(&CSV_BTP);
    
    // free memory used
    free(CSV_BTP.KEYFRAMES);
    free(CSV_BTP.MAT_NAMES);
    free(CSV_BTP.MAT_ANIM_LENGTH);
    free(CSV_BTP.MAT_TEX_SWAP_DATA);
  }
  
  // free anim_file
  free(anim_file);
}


///////////////////////
// read_anim() function
// function to read an anim file based on the anim file type
void read_model(char * model_file_path)
{
  // check model file and load it into memory
  is_model(model_file_path);            
  u1 * model_file = load_file(model_file_path);
  struct BIN_BDL BIN_BDL;
  BUILD_BIN_BDL_STRUCT(&BIN_BDL, model_file);
  PRINT_BIN_BDL_STRUCT(&BIN_BDL, model_file);
  
  // free memory used
  free(model_file);
}


////////////////////////////
// write_csv_anim() function
// function to write a CSV anim file
void write_csv_anim(char * anim_file_path, int anim_type)
{
  // load the anim file into memory
  u1 * anim_file = load_file(anim_file_path);
  
  // decide what to do with the anim file type
  if (anim_type == BIN_BTP_TYPE)
  {
    // create and fill CSV_BTP struct
    struct BIN_BTP BIN_BTP;
    BUILD_BIN_BTP_STRUCT(&BIN_BTP, anim_file);
    WRITE_CSV_BTP(anim_file_path, &BIN_BTP, anim_file);
  }
  
  // free anim_file
  free(anim_file);
}


////////////////////////////
// write_bin_anim() function
// function to write a BIN anim file
void write_bin_anim(char * anim_file_path, int anim_type, char * model_file_path)
{  
  // load the anim file into memory
  u1 * anim_file = load_file(anim_file_path);
  
  // decide what to do with the anim file type
  if (anim_type == CSV_BTP_TYPE)
  {
    // create and fill CSV_BTP struct
    struct CSV_BTP CSV_BTP;
    BUILD_CSV_BTP_STRUCT(&CSV_BTP, anim_file);
    WRITE_BIN_BTP(anim_file_path, &CSV_BTP, model_file_path);
    
    // free memory used
    free(CSV_BTP.KEYFRAMES);
    free(CSV_BTP.MAT_NAMES);
    free(CSV_BTP.MAT_ANIM_LENGTH);
    free(CSV_BTP.MAT_TEX_SWAP_DATA);
  }
  
  // free anim_file
  free(anim_file);
}

==> 1.1.zip: bean/main.c <==
/////////////////
// CUSTOM HEADERS
// ORDER MATTERS !!!
#include "jsystem.h"
#include "CSV/CSV.h"
#include "BDL/BDL.h"
#include "BTP/BIN_BTP.h"
#include "BTP/CSV_BTP.h"


/////////////////////////
// ENUM WITH ACTION TO DO
enum PROGRAM_ACTION {
    
    // nothing to do
    NOTHING_TO_DO,
    
    // checking
    CHECK_ANIM,             // check if anim file has a correct structure
    CHECK_MODEL,            // check if model file has a correct structure
    CHECK_ANIM_WITH_MODEL,  // check if anim file (either CSV or BIN) and binary model file are related
    
    // reading
    READ_ANIM,              // read anim file (either CSV or BIN)
    READ_MODEL,             // read binary model file
    READ_ANIM_WITH_MODEL,   // read both anim file (either CSV or BIN) and binary model file
    
    // converting
    CONVERT_CSV_ANIM_TO_BIN_ANIM,   // convert csv anim file into bin anim file
    CONVERT_BIN_ANIM_TO_CSV_ANIM,   // convert bin anim file into csv anim file
  };


////////////////////////
// VALID COMMAND STRINGS
u1 ANIM_INPUT_ARG[] = "-anim";
u1 MODEL_INPUT_ARG[] = "-model";
u1 READ_FILE_ARG[] = "-read";
u1 CHECK_FILE_ARG[] = "-check";
u1 CONVERT_FILE_ARG[] = "-convert";
u1 CONVERT_TO_CSV[] = "csv";
u1 CONVERT_TO_BIN[] = "bin";


////////////////
// PROGRAM USAGE
// ./a.out -file [JSYSTEM animation file] -model [BDL model] -read/-convert [csv/bin]

// functions only used on main.c
void display_help();
int check_command(int, char * []);
int is_anim(char *);
u4 is_model(char *);
void read_anim(char *);
void read_model(char *);
void write_csv_anim(char *, int);
void write_bin_anim(char *, int, char *);


////////////////
// MAIN FUNCTION
////////////////

int main(int argc, char * argv[])
{ 
  // check command and get program action
  enum PROGRAM_ACTION ACTION = check_command(argc, argv);
  
  enum ANIM_FILE_TYPE ANIM_TYPE = NO_TYPE;
  
  // another epic switch statement
  switch(ACTION)
  {
    ////////////////////////////////////
    // ./a.out -anim iceflick.btp -check
    case(CHECK_ANIM):
      is_anim(argv[2]);     
      break;
    
    //////////////////////////////////
    // ./a.out -model mario.bdl -check
    case(CHECK_MODEL):
      is_model(argv[2]);
      break;
    
    ///////////////////////////////////
    // ./a.out -anim iceflick.btp -read
    case(READ_ANIM):
    
      // read anim file
      read_anim(argv[2]);
      break;
    
    /////////////////////////////////  
    // ./a.out -model mario.bdl -read
    case(READ_MODEL):
      
      // read model
      read_model(argv[2]);
      break; 
    
    /////////////////////////////////////////////////////
    // ./a.out -anim iceflick.btp -model mario.bdl -check
    case(CHECK_ANIM_WITH_MODEL):
      
      // check anim file and model file
      ANIM_TYPE = is_anim(argv[2]);            
      is_model(argv[4]);
      
      // decide what to do with the anim file type
      if (ANIM_TYPE == BIN_BTP_TYPE) {
        if (CHECK_BIN_BTP_WITH_BDL(argv[2], argv[4]) == false)
          exit(1);
      }
      else if (ANIM_TYPE == CSV_BTP_TYPE) {
        if (CHECK_CSV_BTP_WITH_BDL(argv[2], argv[4]) == false)
          exit(1);
      }
      break; 
    
    ////////////////////////////////////////////////////
    // ./a.out -anim iceflick.btp -model mario.bdl -read
    case(READ_ANIM_WITH_MODEL):
      
      // read both files
      read_model(argv[4]);
      read_anim(argv[2]);
      break;
    
    ///////////////////////////////////////////////////////////
    // ./a.out -anim iceflick.btp -convert csv
    // ./a.out -anim iceflick.btp -model mario.bdl -convert csv
    case(CONVERT_BIN_ANIM_TO_CSV_ANIM):
      
      // if model is provided
      // check if both anim and model files are related      
      // if so write the CSV file
      // otherwise just write the CSV BTP
      
      // check anim file
      ANIM_TYPE = is_anim(argv[2]); 
      
      // check if the file is actually valid for conversion
      if (ANIM_TYPE == CSV_BTP_TYPE)
      {
        printf("\nCannot convert CSV file into CSV file.\nTerminating program.\n");
        exit(1);
      }
      
      // model is provided
      if (argc == 7)
      {
        // check model file           
        is_model(argv[4]);
        
        // decide what to do with the anim file type
        if (ANIM_TYPE == BIN_BTP_TYPE) {
          if (CHECK_BIN_BTP_WITH_BDL(argv[2], argv[4]) == false)
            exit(1);
        }
      }
      
      // write csv animation file
      write_csv_anim(argv[2], ANIM_TYPE);
      break;
    
    ///////////////////////////////////////////////////////////////
    // ./a.out -anim iceflick.btp.csv -model mario.bdl -convert bin
    case(CONVERT_CSV_ANIM_TO_BIN_ANIM):
    
      // check anim file
      ANIM_TYPE = is_anim(argv[2]);
      
      // check if the file is actually valid for conversion
      if (ANIM_TYPE == BIN_BTP_TYPE)
      {
        printf("\nCannot convert BIN file into BIN file.\nTerminating program.\n");
        exit(1);
      }
      
      // check model file           
      is_model(argv[4]);
        
      // decide what to do with the anim file type
      if (ANIM_TYPE == CSV_BTP_TYPE)
      {
        if (CHECK_CSV_BTP_WITH_BDL(argv[2], argv[4]) == false)
          exit(1);
      }
      
      // write csv animation file
      write_bin_anim(argv[2], ANIM_TYPE, argv[4]);      
      break;
    
    ///////
    // how?
    default:
      printf("How on earth you managed to get here?\n");
      break;
  }
  
  // program end
  return 0;
}


//////////////////////////
// display_help() function
// function used to display program usage in case
// user inputs wrong information to the program.
// Also terminates the program session.
void display_help()
{
  // display some help  
  printf("Mateee... I dunnae know what to do with that command <:[\n");
  printf("Try again after these command examples:\n");
  
  printf("\n### Check model file\n");
  printf("bean -model mario.bdl -check\n");
  
  printf("\n### Check anim file\n");
  printf("bean -anim iceflick.btp -check\n");
  
  printf("\n### Read model file file\n");
  printf("bean -model mario.bdl -read\n");
  
  printf("\n### Read anim file file\n");
  printf("bean -anim iceflick.btp -read\n");
  
  printf("\n### Convert a BIN anim file into a CSV (NOT providing a model file)\n");
  printf("bean -anim iceflick.btp -convert csv\n");
  
  printf("\n### Read both an anim file and a model file\n");
  printf("bean -anim iceflick.btp -model mario.bdl -read\n");
  
  printf("\n### Check if an anim file and a model file are related\n");
  printf("bean -anim iceflick.btp -model mario.bdl -check\n");
  
  printf("\n### Convert a BIN anim file into a CSV (providing a model file)\n");
  printf("bean -anim iceflick.btp -model mario.bdl -convert csv\n");
  
  printf("\n### Convert a CSV anim file into a BIN (providing a model file)\n");
  printf("bean -anim iceflick.btp.csv -model mario.bdl -convert bin\n");
  
  // incorrect command exit
  exit(1);
}


///////////////////////////
// check_command() function
// function used to check if command 
// entered by the user is valid. Will
// call display_help() in case it isn't.
int check_command(int argc, char * argv[])
{
  // variable to return
  enum PROGRAM_ACTION ACTION;
  
  // huge and epic switch statement >:]
  switch (argc)
  {
    // - checking a single file (anim or model)
    // - reading a single file (anim or model)
    
    // ./a.out -model mario.bdl -check
    // ./a.out -anim iceflick.btp -read
    
    case(4):      
      
      // checking an anim file
      if (compare_strings(argv[1], ANIM_INPUT_ARG) && compare_strings(argv[3], CHECK_FILE_ARG))
        ACTION = CHECK_ANIM;
      // checking a model file
      else if (compare_strings(argv[1], MODEL_INPUT_ARG) && compare_strings(argv[3], CHECK_FILE_ARG))
        ACTION = CHECK_MODEL;
      // reading an anim file
      else if (compare_strings(argv[1], ANIM_INPUT_ARG) && compare_strings(argv[3], READ_FILE_ARG))
        ACTION = READ_ANIM;
      // reading an model file
      else if (compare_strings(argv[1], MODEL_INPUT_ARG) && compare_strings(argv[3], READ_FILE_ARG))
        ACTION = READ_MODEL;
      // invalid command
      else
        ACTION = NOTHING_TO_DO;
        
      break;
    
    // - converting a bin anim file into csv (without providing model file)
    
    // ./a.out -anim iceflick.btp -convert csv
    
    case(5):
      
      // converting a bin anim file into csv
      if (compare_strings(argv[1], ANIM_INPUT_ARG) &&
          compare_strings(argv[3], CONVERT_FILE_ARG) &&
          compare_strings(argv[4], CONVERT_TO_CSV))
        ACTION = CONVERT_BIN_ANIM_TO_CSV_ANIM;
      // invalid command
      else
        ACTION = NOTHING_TO_DO;
    
      break;
    
    // - checking if an anim file and a model file relate
    // - reading both an anim file and a model file (don't check if they relate)
    
    // ./a.out -anim iceflick.btp -model mario.bdl -read
    // ./a.out -anim iceflick.btp -model mario.bdl -check
    
    case(6):
    
      if (compare_strings(argv[1], ANIM_INPUT_ARG) &&
          compare_strings(argv[3], MODEL_INPUT_ARG))
      {
        // read both anim and model files
        if (compare_strings(argv[5], READ_FILE_ARG))
          ACTION = READ_ANIM_WITH_MODEL;
        // check if both anim and model files are related
        else if (compare_strings(argv[5], CHECK_FILE_ARG))
          ACTION = CHECK_ANIM_WITH_MODEL;
        // invalid command
        else
          ACTION = NOTHING_TO_DO;
      }
      // invalid command
      else
        ACTION = NOTHING_TO_DO;
    
      break;
    
    // - converting a binary anim file into a csv file (providing model file)
    // - converting a csv anim file into a bin file (providing model file)
    
    // ./a.out -anim iceflick.btp -model mario.bdl -convert csv
    // ./a.out -anim iceflick.btp.csv -model mario.bdl -convert bin
    
    case(7):
      
      if (compare_strings(argv[1], ANIM_INPUT_ARG) &&
          compare_strings(argv[3], MODEL_INPUT_ARG) &&
          compare_strings(argv[5], CONVERT_FILE_ARG))
      {
        // convert to csv
        if (compare_strings(argv[6], CONVERT_TO_CSV))
          ACTION = CONVERT_BIN_ANIM_TO_CSV_ANIM;
        // convert to bin
        else if (compare_strings(argv[6], CONVERT_TO_BIN))
          ACTION = CONVERT_CSV_ANIM_TO_BIN_ANIM;
        // invalid command
        else
          ACTION = NOTHING_TO_DO;
      }
      // invalid command
      else
        ACTION = NOTHING_TO_DO;
      
      break;
      
    default:
      ACTION = NOTHING_TO_DO;
      break;
  }
  
  if (ACTION == NOTHING_TO_DO)
    display_help();
  
  return ACTION;
}

////////////////////
// is_anim() function
// function used to check if a file is a correct anim file
int is_anim(char * anim_file_path)
{  
  // enum integer to return
  enum ANIM_FILE_TYPE anim_type = NO_TYPE;
  
  // get file extension
  char * file_ext = get_file_ext(anim_file_path);
  
  // CSV file check
  if (compare_strings(file_ext, FILE_EXT[0]))
    anim_type = GET_CSV_TYPE(anim_file_path);
  // BIN BTP check
  else if (compare_strings(file_ext, FILE_EXT[2]))
    anim_type = CHECK_BIN_BTP(anim_file_path);
  
  if (anim_type == NO_TYPE)
  {
    printf("File %s is not a valid anim file.\nCheck the file extension and/or file contents and try again.\n", anim_file_path);
    exit(1);
  }
  
  // otherwise, return file size
  return anim_type;
}


////////////////////
// is_bdl() function
// function used to check if a file is a BDL file
u4 is_model(char * file_path)
{
  // get file size
  u4 bdl_size = CHECK_BDL(file_path);
  // get file extension
  char * file_ext = get_file_ext(file_path);
  
  // even if the file is good, the program won't accept extensionless files
  if (compare_strings(file_ext, FILE_EXT[1]) == false)
    bdl_size = 0;
  
   // CHECK_BDL returned 0 or the file extension is inconsistent
  if (bdl_size == false) {
    printf("File %s is not a valid BDL.\nCheck the file and try again.\n", file_path);
    exit(1);
  }
  
  // if the file is good, return its file size
  return bdl_size;
}


///////////////////////
// read_anim() function
// function to read an anim file based on the anim file type
void read_anim(char * anim_file_path)
{
  // check the anim file type and load it into memory
  int ANIM_TYPE = is_anim(anim_file_path);
  u1 * anim_file = load_file(anim_file_path);
  
  // decide what to do with the anim file type
  if (ANIM_TYPE == BIN_BTP_TYPE)
  {
    // create and fill CSV_BTP struct
    struct BIN_BTP BIN_BTP;
    BUILD_BIN_BTP_STRUCT(&BIN_BTP, anim_file);
    PRINT_BIN_BTP_STRUCT(&BIN_BTP, anim_file);
  }
  else if (ANIM_TYPE == CSV_BTP_TYPE)
  {
    // create and fill CSV_BTP struct
    struct CSV_BTP CSV_BTP;
    BUILD_CSV_BTP_STRUCT(&CSV_BTP, anim_file);
    PRINT_CSV_BTP_STRUCT(&CSV_BTP);
    
    // free memory used
    free(CSV_BTP.KEYFRAMES);
    free(CSV_BTP.MAT_NAMES);
    free(CSV_BTP.MAT_LAST_ANIM_DATA_FRAME);
    free(CSV_BTP.MAT_TEX_SWAP_DATA);
  }
  
  // free anim_file
  free(anim_file);
}


///////////////////////
// read_anim() function
// function to read an anim file based on the anim file type
void read_model(char * model_file_path)
{
  // check model file and load it into memory
  is_model(model_file_path);            
  u1 * model_file = load_file(model_file_path);
  struct BIN_BDL BIN_BDL;
  BUILD_BIN_BDL_STRUCT(&BIN_BDL, model_file);
  PRINT_BIN_BDL_STRUCT(&BIN_BDL, model_file);
  
  // free memory used
  free(model_file);
}


////////////////////////////
// write_csv_anim() function
// function to write a CSV anim file
void write_csv_anim(char * anim_file_path, int anim_type)
{
  // load the anim file into memory
  u1 * anim_file = load_file(anim_file_path);
  
  // decide what to do with the anim file type
  if (anim_type == BIN_BTP_TYPE)
  {
    // create and fill CSV_BTP struct
    struct BIN_BTP BIN_BTP;
    BUILD_BIN_BTP_STRUCT(&BIN_BTP, anim_file);
    WRITE_CSV_BTP(anim_file_path, &BIN_BTP, anim_file);
  }
  
  // free anim_file
  free(anim_file);
}


////////////////////////////
// write_bin_anim() function
// function to write a BIN anim file
void write_bin_anim(char * anim_file_path, int anim_type, char * model_file_path)
{  
  // load the anim file into memory
  u1 * anim_file = load_file(anim_file_path);
  
  // decide what to do with the anim file type
  if (anim_type == CSV_BTP_TYPE)
  {
    // create and fill CSV_BTP struct
    struct CSV_BTP CSV_BTP;
    BUILD_CSV_BTP_STRUCT(&CSV_BTP, anim_file);
    WRITE_BIN_BTP(anim_file_path, &CSV_BTP, model_file_path);
    
    // free memory used
    free(CSV_BTP.KEYFRAMES);
    free(CSV_BTP.MAT_NAMES);
    free(CSV_BTP.MAT_LAST_ANIM_DATA_FRAME);
    free(CSV_BTP.MAT_TEX_SWAP_DATA);
  }
  
  // free anim_file
  free(anim_file);
}
